@@ -44,7 +44,9 @@ static GType _ptr_array_type = G_TYPE_INVALID;
 static JSBool gjs_value_from_g_value_internal(JSContext    *context,
                                               jsval        *value_p,
                                               const GValue *gvalue,
-                                              gboolean      no_copy);
+                                              gboolean      no_copy,
+                                              GSignalQuery *signal_query,
+                                              gint          arg_n);
 
 static void
 closure_marshal(GClosure        *closure,
@@ -114,7 +116,7 @@ closure_marshal(GClosure        *closure,
             no_copy = (signal_query.param_types[i - 1] & G_SIGNAL_TYPE_STATIC_SCOPE) != 0;
         }
 
-        if (!gjs_value_from_g_value_internal(context, &argv[i], gval, no_copy)) {
+        if (!gjs_value_from_g_value_internal(context, &argv[i], gval, no_copy, &signal_query, i)) {
             gjs_debug(GJS_DEBUG_GCLOSURE,
                       "Unable to convert arg %d in order to invoke closure",
                       i);
@@ -402,11 +404,14 @@ gjs_value_to_g_value_internal(JSContext    *context,
         else
             g_value_set_boxed(gvalue, gboxed);
     } else if (g_type_is_a(gtype, G_TYPE_ENUM)) {
-        if (JSVAL_IS_INT(value)) {
+        gint64 value_int64;
+
+        if (gjs_value_to_int64 (context, value, &value_int64)) {
             GEnumValue *v;
 
+            /* See arg.c:_gjs_enum_to_int() */
             v = g_enum_get_value(G_ENUM_CLASS(g_type_class_peek(gtype)),
-                                 JSVAL_TO_INT(value));
+                                 (int)value_int64);
             if (v == NULL) {
                 gjs_throw(context,
                           "%d is not a valid value for enumeration %s",
@@ -423,11 +428,14 @@ gjs_value_to_g_value_internal(JSContext    *context,
             return JS_FALSE;
         }
     } else if (g_type_is_a(gtype, G_TYPE_FLAGS)) {
-        if (JSVAL_IS_INT(value)) {
-            if (!_gjs_flags_value_is_valid(context, gtype, JSVAL_TO_INT(value)))
+        gint64 value_int64;
+
+        if (gjs_value_to_int64 (context, value, &value_int64)) {
+            if (!_gjs_flags_value_is_valid(context, gtype, value_int64))
                 return JS_FALSE;
 
-            g_value_set_flags(gvalue, value);
+            /* See arg.c:_gjs_enum_to_int() */
+            g_value_set_flags(gvalue, (int)value_int64);
         } else {
             gjs_throw(context,
                       "Wrong type %s; flags %s expected",
@@ -512,11 +520,43 @@ gjs_value_to_g_value_no_copy(JSContext    *context,
     return gjs_value_to_g_value_internal(context, value, gvalue, TRUE);
 }
 
+static JSBool
+convert_int_to_enum (JSContext *context,
+                     jsval     *value_p,
+                     GType      gtype,
+                     int        v)
+{
+    double v_double;
+
+    if (v > 0 && v < G_MAXINT) {
+        /* Optimize the unambiguous case */
+        v_double = v;
+    } else {
+        GIBaseInfo *info;
+
+        /* Need to distinguish between negative integers and unsigned integers */
+
+        info = g_irepository_find_by_gtype(g_irepository_get_default(),
+                                           gtype);
+
+        if (info == NULL) /* hope for the best */
+            v_double = v;
+        else
+            v_double = _gjs_enum_from_int ((GIEnumInfo *)info, v);
+
+        g_base_info_unref(info);
+    }
+
+    return JS_NewNumberValue(context, v_double, value_p);
+}
+
 static JSBool
 gjs_value_from_g_value_internal(JSContext    *context,
                                 jsval        *value_p,
                                 const GValue *gvalue,
-                                gboolean      no_copy)
+                                gboolean      no_copy,
+                                GSignalQuery *signal_query,
+                                gint          arg_n)
 {
     GType gtype;
 
@@ -626,9 +666,7 @@ gjs_value_from_g_value_internal(JSContext    *context,
         }
         *value_p = OBJECT_TO_JSVAL(obj);
     } else if (g_type_is_a(gtype, G_TYPE_ENUM)) {
-        int v;
-        v = g_value_get_enum(gvalue);
-        *value_p = INT_TO_JSVAL(v);
+        return convert_int_to_enum(context, value_p, gtype, g_value_get_enum(gvalue));
     } else if (g_type_is_a(gtype, G_TYPE_PARAM)) {
         GParamSpec *gparam;
         JSObject *obj;
@@ -637,6 +675,39 @@ gjs_value_from_g_value_internal(JSContext    *context,
 
         obj = gjs_param_from_g_param(context, gparam);
         *value_p = OBJECT_TO_JSVAL(obj);
+    } else if (signal_query && g_type_is_a(gtype, G_TYPE_POINTER)) {
+        JSBool res;
+        GArgument arg;
+        GIArgInfo *arg_info;
+        GIBaseInfo *obj;
+        GISignalInfo *signal_info;
+        GITypeInfo type_info;
+
+        obj = g_irepository_find_by_gtype(NULL, signal_query->itype);
+        if (!obj) {
+            gjs_throw(context, "Signal argument with GType %s isn't introspectable",
+                      g_type_name(signal_query->itype));
+            return JS_FALSE;
+        }
+
+        signal_info = g_object_info_find_signal((GIObjectInfo*)obj, signal_query->signal_name);
+
+        if (!signal_info) {
+            gjs_throw(context, "Unknown signal.");
+            g_base_info_unref((GIBaseInfo*)obj);
+            return JS_FALSE;
+        }
+        arg_info = g_callable_info_get_arg(signal_info, arg_n - 1);
+        g_arg_info_load_type(arg_info, &type_info);
+
+        arg.v_pointer = g_value_get_pointer(gvalue);
+
+        res = gjs_value_from_g_argument(context, value_p, &type_info, &arg);
+
+        g_base_info_unref((GIBaseInfo*)arg_info);
+        g_base_info_unref((GIBaseInfo*)signal_info);
+        g_base_info_unref((GIBaseInfo*)obj);
+        return res;
     } else if (g_type_is_a(gtype, G_TYPE_POINTER)) {
         gpointer pointer;
 
@@ -678,7 +749,7 @@ gjs_value_from_g_value(JSContext    *context,
                        jsval        *value_p,
                        const GValue *gvalue)
 {
-    return gjs_value_from_g_value_internal(context, value_p, gvalue, FALSE);
+    return gjs_value_from_g_value_internal(context, value_p, gvalue, FALSE, NULL, 0);
 }
 
 __attribute__((constructor)) static void
