@@ -169,14 +169,11 @@ gjs_dbus_proxy_constructor (GType                  type,
                             GObjectConstructParam *construct_params)
 {
     GObject *object;
-    GjsDBusProxy *proxy;
 
     object = (* G_OBJECT_CLASS (gjs_dbus_proxy_parent_class)->constructor) (type,
                                                                             n_construct_properties,
                                                                             construct_params);
 
-    proxy = GJS_DBUS_PROXY(object);
-
     return object;
 }
 
@@ -557,13 +554,13 @@ pending_call_free_data(void *data)
     reply_closure_free(c);
 }
 
-static void
-gjs_dbus_proxy_send_internal(GjsDBusProxy              *proxy,
-                             DBusMessage               *message,
-                             GjsDBusProxyReplyFunc      plain_func,
-                             GjsDBusProxyJsonReplyFunc  json_func,
-                             GjsDBusProxyErrorReplyFunc error_func,
-                             void                      *data)
+void
+gjs_dbus_proxy_send_full(GjsDBusProxy              *proxy,
+                         DBusMessage               *message,
+                         GjsDBusProxyReplyFunc      plain_func,
+                         GjsDBusProxyJsonReplyFunc  json_func,
+                         GjsDBusProxyErrorReplyFunc error_func,
+                         void                      *data)
 {
     ReplyClosure *c;
     DBusPendingCall *pending;
@@ -606,7 +603,7 @@ gjs_dbus_proxy_send(GjsDBusProxy              *proxy,
                     GjsDBusProxyErrorReplyFunc error_func,
                     void                      *data)
 {
-    gjs_dbus_proxy_send_internal(proxy, message, reply_func, NULL, error_func, data);
+    gjs_dbus_proxy_send_full(proxy, message, reply_func, NULL, error_func, data);
 }
 
 static void
@@ -658,7 +655,7 @@ gjs_dbus_proxy_call_json_async (GjsDBusProxy              *proxy,
 
     dbus_message_iter_close_container(&arg_iter, &dict_iter);
 
-    gjs_dbus_proxy_send_internal(proxy, message, NULL, reply_func, error_func, data);
+    gjs_dbus_proxy_send_full(proxy, message, NULL, reply_func, error_func, data);
 
     dbus_message_unref(message);
 }
