@@ -32,12 +32,15 @@
 #include "compat.h"
 
 #include <util/log.h>
+#include <util/glib.h>
 #include <util/error.h>
 
 #include <string.h>
 
 #include <jsapi.h>
 
+#define _GJS_JS_VERSION_DEFAULT "1.8"
+
 static void     gjs_context_dispose           (GObject               *object);
 static void     gjs_context_finalize          (GObject               *object);
 static GObject* gjs_context_constructor       (GType                  type,
@@ -61,6 +64,8 @@ struct _GjsContext {
 
     GjsProfiler *profiler;
 
+    char *jsversion_string;
+
     char **search_path;
 
     unsigned int we_own_runtime : 1;
@@ -82,6 +87,7 @@ static int signals[LAST_SIGNAL];
 
 enum {
     PROP_0,
+    PROP_JS_VERSION,
     PROP_SEARCH_PATH,
     PROP_RUNTIME
 };
@@ -282,7 +288,7 @@ gjs_printerr(JSContext *context,
 static void
 gjs_context_init(GjsContext *js_context)
 {
-
+    js_context->jsversion_string = g_strdup(_GJS_JS_VERSION_DEFAULT);
 }
 
 static void
@@ -298,10 +304,11 @@ gjs_context_class_init(GjsContextClass *klass)
     object_class->get_property = gjs_context_get_property;
     object_class->set_property = gjs_context_set_property;
 
-    pspec = g_param_spec_pointer("search-path",
-                                 "Search path",
-                                 "Path where modules to import should reside",
-                                 G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+    pspec = g_param_spec_boxed("search-path",
+                               "Search path",
+                               "Path where modules to import should reside",
+                               G_TYPE_STRV,
+                               G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
     g_object_class_install_property(object_class,
                                     PROP_SEARCH_PATH,
@@ -316,12 +323,16 @@ gjs_context_class_init(GjsContextClass *klass)
                                     PROP_RUNTIME,
                                     pspec);
 
-    pspec = g_param_spec_boolean("is-load-context",
-                                 "IsLoadContext",
-                                 "Whether this is the load context",
-                                 FALSE,
+    pspec = g_param_spec_string("js-version",
+                                 "JS Version",
+                                 "A string giving the default for the (SpiderMonkey) JavaScript version",
+                                 _GJS_JS_VERSION_DEFAULT,
                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
+    g_object_class_install_property(object_class,
+                                    PROP_JS_VERSION,
+                                    pspec);
+
     gjs_register_native_module("byteArray", gjs_define_byte_array_stuff, 0);
 }
 
@@ -378,6 +389,8 @@ gjs_context_finalize(GObject *object)
         js_context->search_path = NULL;
     }
 
+    g_free(js_context->jsversion_string);
+
     g_static_mutex_lock(&contexts_lock);
     all_contexts = g_list_remove(all_contexts, object);
     g_static_mutex_unlock(&contexts_lock);
@@ -470,9 +483,13 @@ out:
 }
 
 static JSBool
-gjs_locale_to_unicode (JSContext *context,
-                       char      *src,
-                       jsval     *retval)
+gjs_locale_to_unicode (JSContext  *context,
+#ifdef JS_LOCALETOUNICODE_NEEDS_CONST_CHAR
+                       const char *src,
+#else
+                       char       *src,
+#endif
+                       jsval      *retval)
 {
     JSBool success;
     char *utf8;
@@ -509,6 +526,7 @@ gjs_context_constructor (GType                  type,
     GObject *object;
     GjsContext *js_context;
     guint32 options_flags;
+    JSVersion js_version;
 
     object = (* G_OBJECT_CLASS (gjs_context_parent_class)->constructor) (type,
                                                                          n_construct_properties,
@@ -560,15 +578,20 @@ gjs_context_constructor (GType                  type,
     /* set ourselves as the private data */
     JS_SetContextPrivate(js_context->context, js_context);
 
-    /* get all the fancy new language features */
-#define OUR_JS_VERSION JSVERSION_1_8
-    if (JS_GetVersion(js_context->context) != OUR_JS_VERSION) {
+    js_version = JS_StringToVersion(js_context->jsversion_string);
+    /* It doesn't make sense to throw here; just use the default if we
+     * don't know.
+     */
+    if (js_version == JSVERSION_UNKNOWN)
+        js_version = JSVERSION_DEFAULT;
+    /* Set the version if we need to. */
+    if (js_version != JSVERSION_DEFAULT && JS_GetVersion(js_context->context) != js_version) {
         gjs_debug(GJS_DEBUG_CONTEXT,
                   "Changing JavaScript version to %s from %s",
-                  JS_VersionToString(OUR_JS_VERSION),
+                  JS_VersionToString(js_version),
                   JS_VersionToString(JS_GetVersion(js_context->context)));
 
-        JS_SetVersion(js_context->context, OUR_JS_VERSION);
+        JS_SetVersion(js_context->context, js_version);
     }
 
     if (!gjs_init_context_standard(js_context->context))
@@ -661,6 +684,9 @@ gjs_context_get_property (GObject     *object,
     js_context = GJS_CONTEXT (object);
 
     switch (prop_id) {
+    case PROP_JS_VERSION:
+        g_value_set_string(value, js_context->jsversion_string);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -679,11 +705,18 @@ gjs_context_set_property (GObject      *object,
 
     switch (prop_id) {
     case PROP_SEARCH_PATH:
-        js_context->search_path = g_strdupv(g_value_get_pointer(value));
+        js_context->search_path = g_value_dup_boxed(value);
         break;
     case PROP_RUNTIME:
         js_context->runtime = g_value_get_pointer(value);
         break;
+    case PROP_JS_VERSION:
+        g_free(js_context->jsversion_string);
+        if (g_value_get_string (value) == NULL)
+            js_context->jsversion_string = g_strdup(_GJS_JS_VERSION_DEFAULT);
+        else
+            js_context->jsversion_string = g_value_dup_string(value);
+        break;
 
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -691,6 +724,96 @@ gjs_context_set_property (GObject      *object,
     }
 }
 
+/**
+ * gjs_context_scan_buffer_for_js_version:
+ * @buffer: A UTF-8 string
+ * @maxbytes: Maximum number of bytes to scan in buffer
+ *
+ * Given a buffer of JavaScript source code (in UTF-8), look for a
+ * comment in it which tells us which version to enable in the
+ * SpiderMonkey engine.
+ *
+ * The comment is inspired by the Firefox MIME type, see e.g.
+ * https://developer.mozilla.org/en/JavaScript/New_in_JavaScript/1.8
+ *
+ * A valid comment string looks like the following, on its own line:
+ * <literal>// application/javascript;version=1.8</literal>
+ *
+ * Returns: A string suitable for use as the GjsContext::version property.
+ *   If the version is unknown or invalid, %NULL will be returned.
+ */
+const char *
+gjs_context_scan_buffer_for_js_version (const char *buffer,
+                                        gssize      maxbytes)
+{
+    const char *prefix = "// application/javascript;version=";
+    const char *substr;
+    JSVersion ver;
+    char buf[20];
+    gssize remaining_bytes;
+    guint i;
+
+    substr = g_strstr_len(buffer, maxbytes, prefix);
+    if (!substr)
+        return NULL;
+
+    remaining_bytes = maxbytes - ((substr - buffer) + strlen (prefix));
+    /* 20 should give us enough space for all the valid JS version strings; anyways
+     * it's really a bug if we're close to the limit anyways. */
+    if (remaining_bytes < (gssize)sizeof(buf)-1)
+        return NULL;
+
+    buf[sizeof(buf)-1] = '\0';
+    strncpy(buf, substr + strlen (prefix), sizeof(buf)-1);
+    for (i = 0; i < sizeof(buf)-1; i++) {
+        if (buf[i] == '\n') {
+            buf[i] = '\0';
+            break;
+        }
+    }
+
+    ver = JS_StringToVersion(buf);
+    if (ver == JSVERSION_UNKNOWN)
+        return NULL;
+    return JS_VersionToString(ver);
+}
+
+/**
+ * gjs_context_scan_file_for_js_version:
+ * @file_path: (type filename): A file path
+ *
+ * Like gjs_context_scan_buffer_for_js_version(), but will open
+ * the file and use the initial 1024 bytes as a buffer.
+ *
+ * Returns: A string suitable for use as GjsContext::version property.
+ */
+const char *
+gjs_context_scan_file_for_js_version (const char *file_path)
+{
+    char *utf8_buf;
+    guint8 buf[1024];
+    const char *version = NULL;
+    gssize len;
+    FILE *f;
+
+    f = fopen(file_path, "r");
+    if (!f)
+        return NULL;
+
+    len = fread(buf, 1, sizeof(buf)-1, f);
+    fclose(f);
+    if (len < 0)
+        return NULL;
+    buf[len] = '\0';
+
+    utf8_buf = _gjs_g_utf8_make_valid((const char*)buf);
+    version = gjs_context_scan_buffer_for_js_version(utf8_buf, sizeof(buf));
+    g_free(utf8_buf);
+
+    return version;
+}
+
+
 GjsContext*
 gjs_context_new(void)
 {
@@ -705,6 +828,33 @@ gjs_context_new_with_search_path(char** search_path)
                          NULL);
 }
 
+/**
+ * gjs_context_maybe_gc:
+ * @context: a #GjsContext
+ * 
+ * Similar to the Spidermonkey JS_MaybeGC() call which
+ * heuristically looks at JS runtime memory usage and
+ * may initiate a garbage collection. 
+ *
+ * This function always unconditionally invokes JS_MaybeGC(), but
+ * additionally looks at memory usage from the system malloc()
+ * when available, and if the delta has grown since the last run
+ * significantly, also initiates a full JavaScript garbage
+ * collection.  The idea is that since GJS is a bridge between
+ * JavaScript and system libraries, and JS objects act as proxies
+ * for these system memory objects, GJS consumers need a way to
+ * hint to the runtime that it may be a good idea to try a
+ * collection.
+ *
+ * A good time to call this function is when your application
+ * transitions to an idle state.
+ */ 
+void
+gjs_context_maybe_gc (GjsContext  *context)
+{
+    gjs_maybe_gc(context->context);
+}
+
 /**
  * gjs_context_get_all:
  *
