@@ -33,6 +33,7 @@
 #include "jsapi-private.h"
 
 #include <string.h>
+#include <math.h>
 
 GQuark
 gjs_util_error_quark (void)
@@ -236,7 +237,7 @@ gjs_runtime_get_current_context(JSRuntime *runtime)
 
 static JSClass global_class = {
     "GjsGlobal", JSCLASS_GLOBAL_FLAGS,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
@@ -811,6 +812,8 @@ gjs_value_debug_string(JSContext      *context,
         bytes = g_malloc((len + 1) * sizeof(char));
         JS_EncodeStringToBuffer(str, bytes, len);
         bytes[len] = '\0';
+    } else {
+        bytes = g_strdup("[invalid string]");
     }
 #endif
     JS_EndRequest(context);
@@ -908,7 +911,7 @@ gjs_explain_scope(JSContext  *context,
               global, debugstr);
     g_free(debugstr);
 
-    parent = JS_GetScopeChain(context);
+    parent = JS_GetGlobalForScopeChain(context);
     chain = g_string_new(NULL);
     while (parent != NULL) {
         char *debug;
@@ -1516,3 +1519,77 @@ gjs_parse_args (JSContext  *context,
     JS_EndRequest(context);
     return JS_FALSE;
 }
+
+#ifdef __linux__
+static void
+_linux_get_self_process_size (gulong *vm_size,
+                              gulong *rss_size)
+{
+    char *contents;
+    char *iter;
+    gsize len;
+    int i;
+
+    *vm_size = *rss_size = 0;
+
+    if (!g_file_get_contents ("/proc/self/stat", &contents, &len, NULL))
+        return;
+
+    iter = contents;
+    /* See "man proc" for where this 22 comes from */
+    for (i = 0; i < 22; i++) {
+        iter = strchr (iter, ' ');
+        if (!iter)
+            goto out;
+        iter++;
+    }
+    sscanf (iter, " %lu", vm_size);
+    iter = strchr (iter, ' ');
+    if (iter)
+        sscanf (iter, " %lu", rss_size);
+
+ out:
+    g_free (contents);
+}
+
+static gulong linux_rss_trigger;
+#endif
+
+/**
+ * gjs_maybe_gc:
+ *
+ * Low level version of gjs_context_maybe_gc().
+ */
+void
+gjs_maybe_gc (JSContext *context)
+{
+    JS_MaybeGC(context);
+
+#ifdef __linux__
+    {
+        /* We initiate a GC if VM or RSS has grown by this much */
+        gulong vmsize;
+        gulong rss_size;
+
+        _linux_get_self_process_size (&vmsize, &rss_size);
+
+        /* linux_rss_trigger is initialized to 0, so currently
+         * we always do a full GC early.
+         *
+         * Here we see if the RSS has grown by 25% since
+         * our last look; if so, initiate a full GC.  In
+         * theory using RSS is bad if we get swapped out,
+         * since we may be overzealous in GC, but on the
+         * other hand, if swapping is going on, better
+         * to GC.
+         */
+        if (rss_size > linux_rss_trigger) {
+            linux_rss_trigger = (gulong) MIN(G_MAXULONG, rss_size * 1.25);
+            JS_GC(context);
+        } else if (rss_size < (0.75 * linux_rss_trigger)) {
+            /* If we've shrunk by 75%, lower the trigger */
+            linux_rss_trigger = (rss_size * 1.25);
+        }
+    }
+#endif
+}
