@@ -23,7 +23,7 @@
 
 #include "dbus-values.h"
 
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
 #include <gjs/compat.h>
 
 #include <gjs-dbus/dbus.h>
@@ -31,6 +31,185 @@
 
 #include <string.h>
 
+static JSBool
+_gjs_js_one_value_from_dbus_array_dict_entry(JSContext        *context,
+                                             DBusMessageIter  *iter,
+                                             jsval            *value_p)
+{
+    /* Create a dictionary object */
+    JSObject *obj;
+    DBusMessageIter array_iter;
+    jsval key_value, entry_value;
+    JSString *key_str;
+    char *key;
+    JSBool retval = JS_FALSE;
+
+    obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    if (obj == NULL)
+        return JS_FALSE;
+
+    key = NULL;
+    key_value = JSVAL_VOID;
+    entry_value = JSVAL_VOID;
+    key_str = NULL;
+
+    JS_AddObjectRoot(context, &obj);
+    JS_AddValueRoot(context, &key_value);
+    JS_AddValueRoot(context, &entry_value);
+    JS_AddStringRoot(context, &key_str);
+
+    dbus_message_iter_recurse(iter, &array_iter);
+
+    while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
+        DBusMessageIter entry_iter;
+
+        /* Cleanup from previous loop */
+        g_free(key);
+        key = NULL;
+
+        dbus_message_iter_recurse(&array_iter, &entry_iter);
+
+        if (!dbus_type_is_basic(dbus_message_iter_get_arg_type(&entry_iter))) {
+            gjs_throw(context, "Dictionary keys are not a basic type, can't convert to JavaScript");
+            goto out;
+        }
+
+        if (!gjs_js_one_value_from_dbus(context, &entry_iter, &key_value))
+            goto out;
+
+        key_str = JS_ValueToString(context, key_value);
+        if (key_str == NULL) {
+            gjs_throw(context, "Couldn't convert value to string");
+            goto out;
+        }
+        if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(key_str), &key))
+            goto out;
+
+        dbus_message_iter_next(&entry_iter);
+
+        gjs_debug_dbus_marshal("Defining dict entry %s in jsval dict", key);
+
+        if (!gjs_js_one_value_from_dbus(context, &entry_iter, &entry_value))
+            goto out;
+
+        if (!JS_DefineProperty(context, obj,
+                               key, entry_value,
+                               NULL, NULL, JSPROP_ENUMERATE))
+            goto out;
+
+        dbus_message_iter_next(&array_iter);
+    }
+
+    *value_p = OBJECT_TO_JSVAL(obj);
+    retval = JS_TRUE;
+ out:
+    g_free(key);
+    JS_RemoveObjectRoot(context, &obj);
+    JS_RemoveValueRoot(context, &key_value);
+    JS_RemoveValueRoot(context, &entry_value);
+    JS_RemoveStringRoot(context, &key_str);
+    return retval;
+}
+
+static JSBool
+_gjs_js_one_value_from_dbus_array_byte(JSContext       *context,
+                                       DBusMessageIter *iter,
+                                       jsval           *value_p)
+{
+    /* byte arrays go to a string */
+    const char *v_BYTES;
+    int n_bytes;
+    DBusMessageIter array_iter;
+
+    dbus_message_iter_recurse(iter, &array_iter);
+    dbus_message_iter_get_fixed_array(&array_iter,
+                                      &v_BYTES, &n_bytes);
+
+    return gjs_string_from_binary_data(context, v_BYTES, n_bytes, value_p);
+}
+
+static JSBool
+_gjs_js_one_value_from_dbus_struct(JSContext       *context,
+                                   DBusMessageIter *iter,
+                                   jsval           *value_p)
+{
+    JSObject *obj;
+    DBusMessageIter struct_iter;
+    int index;
+    jsval prop_value;
+    JSBool retval = JS_FALSE;
+
+    obj = JS_NewArrayObject(context, 0, NULL);
+    if (obj == NULL)
+        return JS_FALSE;
+
+    prop_value = JSVAL_VOID;
+    JS_AddValueRoot(context, &prop_value);
+
+    dbus_message_iter_recurse(iter, &struct_iter);
+    index = 0;
+    while (dbus_message_iter_get_arg_type(&struct_iter) != DBUS_TYPE_INVALID) {
+
+        if (!gjs_js_one_value_from_dbus(context, &struct_iter, &prop_value))
+            goto out;
+
+        if (!JS_DefineElement(context, obj,
+                              index, prop_value,
+                              NULL, NULL, JSPROP_ENUMERATE))
+            goto out;
+
+        dbus_message_iter_next(&struct_iter);
+        index++;
+    }
+    *value_p = OBJECT_TO_JSVAL(obj);
+    retval = JS_TRUE;
+ out:
+    JS_RemoveValueRoot(context, &prop_value);
+    return retval;
+}
+
+static JSBool
+_gjs_js_one_value_from_dbus_array_other(JSContext       *context,
+                                        DBusMessageIter *iter,
+                                        jsval           *value_p)
+{
+    JSObject *obj;
+    jsval prop_value;
+    DBusMessageIter array_iter;
+    int index;
+    JSBool retval = JS_FALSE;
+
+    obj = JS_NewArrayObject(context, 0, NULL);
+    if (obj == NULL)
+        return JS_FALSE;
+
+    prop_value = JSVAL_VOID;
+
+    JS_AddObjectRoot(context, &obj);
+    JS_AddValueRoot(context, &prop_value);
+    dbus_message_iter_recurse(iter, &array_iter);
+    index = 0;
+    while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
+        if (!gjs_js_one_value_from_dbus(context, &array_iter, &prop_value))
+            goto out;
+
+        if (!JS_DefineElement(context, obj,
+                              index, prop_value,
+                              NULL, NULL, JSPROP_ENUMERATE))
+            goto out;
+
+        dbus_message_iter_next(&array_iter);
+        index++;
+    }
+
+    *value_p = OBJECT_TO_JSVAL(obj);
+    retval = JS_TRUE;
+ out:
+    JS_RemoveObjectRoot(context, &obj);
+    JS_RemoveValueRoot(context, &prop_value);
+    return retval;
+}
+
 JSBool
 gjs_js_one_value_from_dbus(JSContext       *context,
                            DBusMessageIter *iter,
@@ -47,163 +226,17 @@ gjs_js_one_value_from_dbus(JSContext       *context,
 
     switch (arg_type) {
     case DBUS_TYPE_STRUCT:
-        {
-            JSObject *obj;
-            DBusMessageIter struct_iter;
-            int index;
-
-            obj = JS_NewArrayObject(context, 0, NULL);
-            if (obj == NULL)
-                return JS_FALSE;
-
-            dbus_message_iter_recurse(iter, &struct_iter);
-            index = 0;
-            while (dbus_message_iter_get_arg_type(&struct_iter) != DBUS_TYPE_INVALID) {
-                jsval prop_value;
-
-                prop_value = JSVAL_VOID;
-                JS_AddValueRoot(context, &prop_value);
-                if (!gjs_js_one_value_from_dbus(context, &struct_iter, &prop_value)) {
-                    JS_RemoveValueRoot(context, &prop_value);
-                    return JS_FALSE;
-                }
-
-                if (!JS_DefineElement(context, obj,
-                                      index, prop_value,
-                                      NULL, NULL, JSPROP_ENUMERATE)) {
-                    JS_RemoveValueRoot(context, &prop_value);
-                    return JS_FALSE;
-                }
-
-                JS_RemoveValueRoot(context, &prop_value);
-                dbus_message_iter_next(&struct_iter);
-                index ++;
-            }
-            *value_p = OBJECT_TO_JSVAL(obj);
-        }
-        break;
+        return _gjs_js_one_value_from_dbus_struct(context, iter, value_p);
     case DBUS_TYPE_ARRAY:
         {
             int elem_type = dbus_message_iter_get_element_type(iter);
 
             if (elem_type == DBUS_TYPE_DICT_ENTRY) {
-                /* Create a dictionary object */
-                JSObject *obj;
-                DBusMessageIter array_iter;
-
-                obj = JS_ConstructObject(context, NULL, NULL, NULL);
-                if (obj == NULL)
-                    return JS_FALSE;
-
-                JS_AddObjectRoot(context, &obj);
-                dbus_message_iter_recurse(iter, &array_iter);
-                while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
-                    DBusMessageIter entry_iter;
-                    jsval key_value, entry_value;
-                    JSString *key_str;
-                    const char *key;
-
-                    dbus_message_iter_recurse(&array_iter, &entry_iter);
-
-                    if (!dbus_type_is_basic(dbus_message_iter_get_arg_type(&entry_iter))) {
-                        gjs_throw(context, "Dictionary keys are not a basic type, can't convert to JavaScript");
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    entry_value = JSVAL_VOID;
-                    JS_AddValueRoot(context, &entry_value);
-                    if (!gjs_js_one_value_from_dbus(context, &entry_iter, &key_value)) {
-                        JS_RemoveValueRoot(context, &key_value);
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    key_str = JS_ValueToString(context, key_value);
-                    JS_AddStringRoot(context, &key_str);
-                    key = JS_GetStringBytes(key_str);
-
-                    dbus_message_iter_next(&entry_iter);
-
-                    gjs_debug_dbus_marshal("Defining dict entry %s in jsval dict", key);
-
-                    entry_value = JSVAL_VOID;
-                    JS_AddValueRoot(context, &entry_value);
-                    if (!gjs_js_one_value_from_dbus(context, &entry_iter, &entry_value)) {
-                        JS_RemoveValueRoot(context, &key_value);
-                        JS_RemoveStringRoot(context, &key_str);
-                        JS_RemoveValueRoot(context, &entry_value);
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    if (!JS_DefineProperty(context, obj,
-                                           key, entry_value,
-                                           NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveValueRoot(context, &key_value);
-                        JS_RemoveStringRoot(context, &key_str);
-                        JS_RemoveValueRoot(context, &entry_value);
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    JS_RemoveValueRoot(context, &key_value);
-                    JS_RemoveStringRoot(context, &key_str);
-                    JS_RemoveValueRoot(context, &entry_value);
-                    dbus_message_iter_next(&array_iter);
-                }
-
-                *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveObjectRoot(context, &obj);
+                return _gjs_js_one_value_from_dbus_array_dict_entry(context, iter, value_p);
             } else if (elem_type == DBUS_TYPE_BYTE) {
-                /* byte arrays go to a string */
-                const char *v_BYTES;
-                int n_bytes;
-                DBusMessageIter array_iter;
-
-                dbus_message_iter_recurse(iter, &array_iter);
-                dbus_message_iter_get_fixed_array(&array_iter,
-                                                  &v_BYTES, &n_bytes);
-
-                if (!gjs_string_from_binary_data(context, v_BYTES, n_bytes, value_p))
-                    return JS_FALSE;
+                return _gjs_js_one_value_from_dbus_array_byte(context, iter, value_p);
             } else {
-                JSObject *obj;
-                DBusMessageIter array_iter;
-                int index;
-
-                obj = JS_NewArrayObject(context, 0, NULL);
-                if (obj == NULL)
-                    return JS_FALSE;
-
-                JS_AddObjectRoot(context, &obj);
-                dbus_message_iter_recurse(iter, &array_iter);
-                index = 0;
-                while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
-                    jsval prop_value;
-
-                    prop_value = JSVAL_VOID;
-                    JS_AddValueRoot(context, &prop_value);
-                    if (!gjs_js_one_value_from_dbus(context, &array_iter, &prop_value)) {
-                        JS_RemoveValueRoot(context, &prop_value);
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    if (!JS_DefineElement(context, obj,
-                                          index, prop_value,
-                                          NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveValueRoot(context, &prop_value);
-                        JS_RemoveObjectRoot(context, &obj);
-                        return JS_FALSE;
-                    }
-
-                    JS_RemoveValueRoot(context, &prop_value);
-                    dbus_message_iter_next(&array_iter);
-                    index ++;
-                }
-                *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveObjectRoot(context, &obj);
+                return _gjs_js_one_value_from_dbus_array_other(context, iter, value_p);
             }
         }
         break;
@@ -212,54 +245,49 @@ gjs_js_one_value_from_dbus(JSContext       *context,
             dbus_bool_t v_BOOLEAN;
             dbus_message_iter_get_basic(iter, &v_BOOLEAN);
             *value_p = BOOLEAN_TO_JSVAL(v_BOOLEAN);
+            return JS_TRUE;
         }
         break;
     case DBUS_TYPE_BYTE:
         {
             unsigned char v_BYTE;
             dbus_message_iter_get_basic(iter, &v_BYTE);
-            if (!JS_NewNumberValue(context, v_BYTE, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_BYTE, value_p);
         }
         break;
     case DBUS_TYPE_INT32:
         {
             dbus_int32_t v_INT32;
             dbus_message_iter_get_basic(iter, &v_INT32);
-            if (!JS_NewNumberValue(context, v_INT32, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_INT32, value_p);
         }
         break;
     case DBUS_TYPE_UINT32:
         {
             dbus_uint32_t v_UINT32;
             dbus_message_iter_get_basic(iter, &v_UINT32);
-            if (!JS_NewNumberValue(context, v_UINT32, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_UINT32, value_p);
         }
         break;
     case DBUS_TYPE_INT64:
         {
             dbus_int64_t v_INT64;
             dbus_message_iter_get_basic(iter, &v_INT64);
-            if (!JS_NewNumberValue(context, v_INT64, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_INT64, value_p);
         }
         break;
     case DBUS_TYPE_UINT64:
         {
             dbus_uint64_t v_UINT64;
             dbus_message_iter_get_basic(iter, &v_UINT64);
-            if (!JS_NewNumberValue(context, v_UINT64, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_UINT64, value_p);
         }
         break;
     case DBUS_TYPE_DOUBLE:
         {
             double v_DOUBLE;
             dbus_message_iter_get_basic(iter, &v_DOUBLE);
-            if (!JS_NewNumberValue(context, v_DOUBLE, value_p))
-                return JS_FALSE;
+            return JS_NewNumberValue(context, v_DOUBLE, value_p);
         }
         break;
     case DBUS_TYPE_OBJECT_PATH:
@@ -269,9 +297,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
 
             dbus_message_iter_get_basic(iter, &v_STRING);
 
-            if (!gjs_string_from_utf8(context, v_STRING, -1, value_p))
-                return JS_FALSE;
-
+            return gjs_string_from_utf8(context, v_STRING, -1, value_p);
         }
         break;
 
@@ -287,6 +313,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
 
     case DBUS_TYPE_INVALID:
         *value_p = JSVAL_VOID;
+        return JS_TRUE;
         break;
 
     default:
@@ -296,8 +323,6 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                      arg_type);
         return JS_FALSE;
     }
-
-    return JS_TRUE;
 }
 
 JSBool
@@ -785,7 +810,7 @@ append_dict(JSContext         *context,
         char *name;
         jsval propval;
         DBusMessageIter entry_iter;
-        const char *value_signature;
+        char *value_signature;
 
         if (!JS_IdToValue(context, prop_id, &nameval))
             return JS_FALSE;
@@ -807,7 +832,7 @@ append_dict(JSContext         *context,
                                     JSVAL_TO_OBJECT(prop_signatures),
                                     name, &signature_value);
             if (signature_value != JSVAL_VOID) {
-                value_signature = gjs_string_get_ascii_checked(context,
+                value_signature = gjs_string_get_ascii(context,
                                                                signature_value);
                 if (value_signature == NULL) {
                     return JS_FALSE;
@@ -815,8 +840,10 @@ append_dict(JSContext         *context,
             }
         }
 
-        if (!gjs_object_require_property(context, props, "DBus append_dict", name, &propval))
+        if (!gjs_object_require_property(context, props, "DBus append_dict", name, &propval)) {
+            g_free(value_signature);
             return JS_FALSE;
+        }
 
         gjs_debug_dbus_marshal(" Adding property %s",
                                name);
@@ -827,6 +854,7 @@ append_dict(JSContext         *context,
         if (JSVAL_IS_NULL(propval)) {
             gjs_throw(context, "Property '%s' has a null value, can't send over dbus",
                       name);
+            g_free(value_signature);
             return JS_FALSE;
         }
 
@@ -855,6 +883,7 @@ append_dict(JSContext         *context,
                 return JS_FALSE;
 
             dbus_message_iter_close_container(&entry_iter, &variant_iter);
+            g_free(value_signature);
         } else {
             if (!gjs_js_one_value_to_dbus(context, propval, &entry_iter,
                                           &dict_value_sig_iter))
